
;     *** Play one game ***

; In:   [Level_Seg] = Game to play
;
; Out:  AL = 0 -> Finished!
;       AL = 1 -> Game over
;       AL = 2 -> Goto menu
;
;       AL = -1 -> Error in level
;       AL = -2 -> Keyb error
;
; Regs: ALL!

Proc    Play_Game        
        mov     cx,33*3                 ; Setup game palete
        mov     si,offset Pal
        xor     bl,bl
        call    Write_Col

        xor     esi,esi                 ; Clear upper word
        xor     edi,edi

        mov     [Dword ds:Score],0      ; Setup data
        mov     [Byte ds:Men],3

        mov     ax,[Word ds:Level_Seg]  ; Get segment to game
        mov     es,ax
        mov     si,4

Game_Loop:
        movzx   bx,[Byte ds:Level_Num]  ; Get level to play
        dec     bx                      ; Zero adjust
        add     bx,bx                   ; Word table
        mov     ax,[Word es:si+bx]      ; Get segment to level
        cmp     ax,-1                   ; Last level?
        jne     @@10
        inc     al                      ; AL = 0, finished!
        jmp     @@90

@@10:   add     ax,[Word ds:Level_Seg]
        call    Play_Level              ; AX = Level to play

        cmp     al,0                    ; Level finished?
        jne     @@20
        inc     [Byte ds:Level_Num]     ; Goto next level
        jmp     Game_Loop


@@20:   cmp     al,1                    ; Dead?
        jne     @@99
        cmp     [Byte ds:Bonus],0       ; Bonus life?
        je      @@25
        inc     [Byte ds:Level_Num]     ; Yes, next level
        jmp     Game_Loop               ; Don't decrease men

@@25:   dec     [Byte ds:Men]
        jnz     Game_Loop               ; AL = 1 -> Game over

@@90:
@@99:   ret                             ; Return AL value from PLAY_LEVEL

EndP    Play_Game






; Setup one level and play it

; In:   AX = Level to play
;
; Out:  AL = 0 -> Level finished
;       AL = 1 -> ESC pressed
;       AL = 2 -> Goto menu
;
;       AL = -1 -> Error in map
;       AL = -2 -> Keyb error
;
; Regs: All - SI,ES

Proc    Play_Level
        push    es si                   ; (!OBS!)


; ES:SI = Level to play
        mov     es,ax
        xor     si,si


; Clear out old level data
        mov     di,Offset D_Count
        mov     cx,Level_Clear / 4
        xor     eax,eax
@@30:   mov     [Dword ds:di],eax
        add     di,4
        loop    @@30


; *** Setup new level data ***

; Get initial random number (ripped...)
        cli
        mov     al,0                    ; Get seconds
        out     70h,al
        in      al,71h
        shl     eax,8
        mov     al,2                    ; Get minute
        out     70h,al
        in      al,71h
        sti
        shl     eax,8
        xor     eax,[Dword fs:046Ch]    ; throw in number of ticks
        not     eax
        mov     [Dword ds:Random_Number],eax

        mov     [Word ds:Info_String],Offset Status_String
        movzx   ax,[Byte es:si+2]       ; Setup time
        inc     ax
        shl     ax,4                    ; * 16
        mov     [Word ds:Time],ax
        mov     al,[Byte es:si]         ; H
        mov     [Byte ds:Map_H],al
        mov     bl,[Byte es:si+1]       ; V
        mov     [Byte ds:Map_V],bl
        mul     bl                      ; AX = H * V
        mov     [Word ds:Map_Size],ax
        push    ax                      ; Save map size
        mov     [Byte ds:Guy_Last],22
        mov     [Byte ds:Green_All],1
        mov     [Byte ds:Time_Color],27 ; Reset to green

        add     si,3                    ; Copy new level data
        mov     di,Offset D_Take
        mov     cx,5
@@35:   mov     eax,[Dword es:si]
        mov     [Dword ds:di],eax
        add     si,4
        add     di,4
        loop    @@35


; Copy Level to map 1 (SI = Level data)
        mov     di,Offset Map1
        pop     cx                      ; CX = Map size
        shr     cx,2                    ; Convert to Dword
        inc     cx                      ; One extra
@@38:   mov     eax,[Dword es:si]
        mov     [Dword ds:di],eax
        add     si,4
        add     di,4
        dec     cx
        jnz     @@38


; *** Write level info to screen ***
        push    ds                      ; ES = Data
        pop     es

        mov     si,Offset Level_Name    ; Setup level name
        mov     di,Str_Name
        mov     cx,13
        rep     movsb
        mov     di,Str_Level            ; Setup level number
        movzx   eax,[Byte ds:Level_Num]
        mov     cx,2
        call    Bin_To_ASCII
        mov     di,Str_Men              ; Setup men left
        movzx   eax,[Byte ds:Men]
        mov     cx,2
        call    Bin_To_ASCII

        call    Clear_Screen            ; ES = Screen

        Prepare_Text                    ; Write level string
        mov     [Byte ds:Char_Space],2
        mov     bp,Offset Level_Str
        mov     di,X_Byte*120 + 7
        call    Write_Text

        cmp     [Byte ds:Bonus],0
        je      @@39
        mov     bp,Offset Bonus_Str     ; Write 'Bonus life' string        
        mov     di,X_Byte*60 + 29
        call    Write_Text

@@39:   mov     [Byte ds:Char_Space],3  ; Reset

        xor     al,al
        mov     si,Offset Last_Key
        mov     [Byte ds:si],al
@@60:   cmp     [Byte ds:si],al         ; Wait for key press
        je      @@60
        mov     [Byte ds:si],al
        mov     [Byte ds:ESC_Flagg],al
        mov     [Byte ds:Playing],1


;============================================================================
 Main_Loop:
;============================================================================
        cmp     [Byte ds:Update],0      ; Sync with timer interupt
        jz      Main_Loop
        dec     [Byte ds:Update]

        cmp     [Byte ds:Pause],1       ; Only update screen if pause is on
        je      @@Update_Screen

        test    [Byte ds:Finished],80h  ; Game over?
        jnz     @@One_Tick              ; Yes, jump


; Reached goal? --> Count down time...
@@40:   mov     ax,[Word ds:Time]
        cmp     [Byte ds:Finished],3    ; Shall we count down time?
        jb      @@50                    ; No, jump
        sub     ax,16*2                 ; Count down (AX = Time)
        jns     @@45
        xor     al,al                   ; Next level
        cmp     [Byte ds:Bonus],0       ; Bonus life?
        je      @@99
        inc     [Byte ds:Men]           ; Yes, extra life
        jmp     @@99
@@45:   mov     [Word ds:Time],ax       ; New time
        mov     eax,[Dword ds:Score]    ; EAX = Old score
        add     [Dword ds:Score],10*2   ; 10 Points for each second left
        mov     ecx,[Dword ds:Score]    ; ECX = New score

        call    _Extra                  ; Check for extra life
        je      @@Update_Screen
        mov     [Byte ds:Extra_Life],60 ; Extra life!
        inc     [Byte ds:Men]
        jmp     @@Update_Screen


; Test for time up...
@@50:   cmp     ax,10                   ; AX = Time
        jae     @@One_Tick

; Sorry, time is up.
        mov     [Word ds:Info_String],Offset TimeUp_Str
        mov     [Byte ds:Char_Space],2
        or      [Byte ds:Finished],80h
        jmp     @@One_Tick


; Don't tick, just update screen
@@Update_Screen:        
        mov     ax,Data_Seg             ; Draw screen and status line
        mov     es,ax
        jmp     @@20


; Handle the green stuff
@@One_Tick:
        mov     [Byte ds:Green_Count],0 ; Reset counter
        cmp     [Byte ds:Green_Stop],0  ; Check if the green stuff may be
        je      @@10                    ; traped
        inc     [Byte ds:Green_Traped]  ; It may be traped

@@10:   cmp     [Byte ds:Green_All],200 ; Check if we shall transform to
        jb      @@11                    ; bolders
        mov     [Byte ds:Green_Transform],6


; Do the moving on the virtual map
@@11:   cmp     [Byte ds:Update2],0     ; Delay game speed
        je      @@Draw
        dec     [Byte ds:Update2]
        mov     [Byte ds:Guy_Alive],1
        call    Move_Map
        xor     [Byte ds:Curr_Map],1    ; Next map

        cmp     [Byte ds:Guy_Alive],0   ; Is guy "missing"?
        je      @@14
        test    [Byte ds:Finished],80h  ; Is he reported dead?
        jnz     @@14

; No, kill him!
        mov     ax,Offset Dead_Str
        cmp     [Byte ds:Men],1
        jne     @@19
        mov     ax,Offset GameOver_Str
@@19:   mov     [Word ds:Info_String],ax
        mov     [Byte ds:Char_Space],2
        or      [Byte ds:Finished],80h


; Green stuff again
@@14:   cmp     [Byte ds:Green_Stop],0          ; Still stoped after handling
        je      @@20                            ; the map?
        cmp     [Byte ds:Green_Traped],0        ; Where we traped before
        je      @@20                            ; handeling the map?
        inc     [Byte ds:Green_Traped2]         ; Yes, we are traped
        cmp     [Byte ds:Green_Traped2],30      ; Wait a short delay just to
        jb      @@20                            ; be sure
        mov     [Byte ds:Green_Transform],9     ; Transform to dimonds


; Setup status line
@@20:   Setup_Status
        mov     ax,0A000h               ; ES = Screen again
        mov     es,ax


; Draw the screen
@@Draw: mov     bp,offset Map1          ; Assume we shall draw map 1
        cmp     [Byte ds:Curr_Map],0    ; Correct?
        je      @@12
        mov     bp,Offset Map2          ; Draw Map2

@@12:   mov     di,Page_1               ; Do the page flipping
        cmp     [Byte ds:Curr_Page],1
        je      @@13
        mov     di,Page_2

@@13:   push    di
        call    Draw_Screen             ; Changes DS
        pop     di
        
; Write status line
        Prepare_Text
        mov     bp,[Word gs:Info_String]
        cmp     bp,Offset Status_String         ; Non't flash info string
        je      @@80
        cmp     [Byte gs:Frame3],1              ; Flash all other!
        ja      @@80
        mov     bp,Offset Nul_Str

@@80:   sub     di,X_Byte*12
        push    di
        add     di,12/4
        call    Write_Text

        pop     bx
        call    Write_SA                ; Flipp page
        xor     [Byte gs:Curr_Page],1   ; Next page

        
        xor     bl,bl                   ; Set normal write mode
        call    Set_W_Mode
        mov     ax,Data_Seg
        mov     ds,ax

; Check if we are finished
        xor     ah,ah

        mov     al,1                    ; ESC pressed?
        cmp     [Byte ds:ESC_Flagg],1
        jne     @@18

        cmp     [Byte ds:Finished],3    ; Count down time?
        jb      @@99
        test    [Byte ds:Finished],80h  ; Game over?
        jnz     @@99

; Pressed ESC, so add time to score
        mov     ax,[Word ds:Time]
        shr     ax,4                    ; AL = Time left
        mov     ah,10                   ; 10 points for each second
        mul     ah                      ; AX = Extra points
        movzx   eax,ax
        mov     ecx,[Dword ds:Score]
        push    ecx
        add     [Dword ds:Score],eax
        add     ecx,eax                 ; New score
        pop     eax                     ; Old score
        call    _Extra                  ; Check for extra life
        je      @@82        
        inc     [Byte ds:Men]

@@82:   xor     al,al                   ; Finished level
        jmp     @@99


@@18:   inc     al                      ; TAB-ESC pressed?
        cmp     [Byte ds:ESC_Flagg],2
        je      @@99

        mov     al,-2                   ; Keyboard error?
        cmp     [Byte ds:Keyb_Error],ah
        jne     @@99

        jmp     Main_Loop               ; Continue


; Return from level
@@99:   mov     [Byte ds:Playing],0
        pop     si es
        ret
;============================================================================
EndP    Play_Level
