
; String and parameter functions...


; Get_Param     : Call before using One_Param or One_Options
; One_Param     : Get one paramter from the command line
; Str_Write     : Write string to standard output. ESI = Str
; Str_Write2    : ESI = Str, CX = Length
; New_Line      : Move currsor to next line
; Write_Dec     : Print a decimal number to the screen
; Write_Hex     : Print a hex number to the screen




SEGMENT         BigSeg
ASSUME          CS:BigSeg




; Print an number to the screen

; In:   EAX = Number
;       CX = Digits
;
; Out:  nothing
;
; Regs: none

Proc    Write_Dec
        pushad
        mov     ebx,10                  ; Base = 10
        jmp     @@00

; Entry for Write_Hex routine...
Write_Hex:
        pushad
        mov     ebx,16                  ; Base = 16

@@00:   xor     si,si
        mov     edi,Offset Tmp_Str

@@10:   xor     edx,edx                 ; EDX:EAX = EAX
        div     ebx                     ; EAX = Answer, EDX <- Remainder
        cmp     dl,10                   ; Hex digit?
        jb      @@15
        add     dl,'A'-10               ; Convert to A B C D E F
        jmp     @@16

@@15:   or      dl,'0'                  ; Convert digit to ASCII 0 - 9
@@16:   push    edx                     ; Save digit
        inc     si                      ; Count digit
        loop    @@10

        mov     cx,si                   ; CX = Digit on stack
@@20:   pop     eax                     ; al <-- Digit
        stosb
        loop    @@20

        cmp     bx,10                   ; Add hex indicator 'h'
        je      @@90
        mov     al,'h'
        stosb

@@90:   mov     [Byte ds:edi],0         ; Str terminator
        mov     esi,Offset Tmp_Str      ; Write string
        call    Str_Write

        popad
        ret
EndP    Write_Dec




; Get one param (In uppercase)

; In:   CX = Param number
;
; Out:  ESI = ASCIIZ string
;
; Regs: AL CX ESI

Proc    One_Param
        mov     esi,Offset Param_Str
        dec     cx                      ; Zero based!
        jcxz    @@99                    ; CX = Zero --> SI = First param!

@@10:   lodsb                           ; Scan for zero terminator
        or      al,al
        jnz     @@10
        loop    @@10                    ; Do it CX times

@@99:   ret
EndP    One_Param



; Write ASCIIZ string to std out

; In:   ESI = String
;
; Out:  nothing
;
; Regs: AX

Proc    Str_Write
        push    ebx ecx edx
        call    Str_Length              ; CX = Length

        mov     bx,1                    ; Standar out handle
        mov     edx,esi
        mov     ah,040h                 ; Write to file or device
        int     21h

        pop     edx ecx ebx
        ret
EndP    Str_Write



; ESI = String --> CX = Length
Proc    Str_Length
        push    esi
        mov     cx,0FFFFh               ; Max string length

@@10:   lodsb
        or      al,al
        loopnz  @@10

        not     cx
        dec     cx

        pop     esi
        ret
EndP    Str_Length



; Moves one line down
Proc    New_Line
        push    edx
        mov     ah,2
        mov     dl,CR
        int     21h
        mov     dl,LF
        int     21h
        pop     edx
        ret
EndP    New_Line





; First thing to call. BX = Number of parameters
Proc    Get_Param
        mov     ax,0EE02h               ; Get DOS32 address information
        int     31h
        add     esi,80h                 ; ESI = Paramter string
        mov     edi,Offset Param_Str
        xor     bx,bx                   ; Parameter count
        xor     ecx,ecx
        mov     ah,1

        mov     cl,[Byte es:esi]        ; Get param length
        jcxz    @@99
        call    _UpperCase              ; Convert to uppercase
        inc     esi

        xor     ah,ah                   ; Find first char

@@20:   mov     al,[Byte ds:esi]        ; Get char
        inc     esi
        call    _Check                  ; Is it a seperator
        jne     @@30                    ; No, jump
        dec     cx                      ; Finished?
        jz      @@90
        or      ah,ah                   ; Have we added the zero?
        jz      @@20                    ; Yes, skipp it
        xor     ax,ax                   ; No, write it now
        inc     bl                      ; Param count
        inc     cx                      ; Don't DEC twice!
        jmp     @@31

@@30:   mov     ah,1                    ; Set new param flagg
@@31:   mov     [Byte ds:edi],al        ; Write copy to string
        inc     edi
        loopd   @@20                    ; Do all chars in string

        mov     [Byte ds:edi],0         ; Write last zero

@@90:   inc     bl                      ; Count last param
@@99:   mov     [Byte ds:Num_Param],bl
        ret
EndP    Get_Param



; ZF = 1 --> AL = Seperator (Jump Equal)
Proc    _Check
        cmp     al,020h                 ; Space?
        je      @@99
        cmp     al,09h                  ; TAB?

@@99:   ret
EndP    _Check




; Converts a string to uppercase
Proc    _UpperCase
        push    ecx esi
        inc     esi

@@10:   mov     al,[Byte ds:esi]
        cmp     al,'a'
        jb      @@20
        cmp     al,'z'
        ja      @@20
        sub     al,'a'-'A'
        mov     [Byte ds:esi],al

@@20:   inc     esi
        loop    @@10

@@99:   pop     esi ecx
        ret
EndP    _UpperCase









EndS            BigSeg




SEGMENT         U_DATA

Num_Param       db      ?
Param_Str       db      129 Dup (?)
Tmp_Str         db      12 Dup (?)

EndS            U_DATA
